/*
 *  DUMBparse.c
 *
 *  Created by Herbert Poetzl on Tue Feb 05 2002.
 *  Copyright (c) 2002 H.Poetzl. 
 *
 *  This file is part of MultiTerm.
 *
 *  MultiTerm is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  MultiTerm is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MultiTerm; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "DUMBparse.h"

#include "parserdef.h"
#include "tstoredef.h"

#define	PINFO	parser->info.pass
#define	PVARS	parser->vars.pass


int	dumb_init(
        TEParser *parser)
{
        PINFO.max_buffer = 512;
        parser->nextstate = &dumb_nextstate;
        parser->transition = &dumb_transition;
        parser->unbuffer = &dumb_unbuffer;
        parser->reset = &dumb_reset;
        return 512;
}

void	dumb_reset(
        TEParser *parser)
{
        PVARS.count = 0;
        PVARS.count = 0;
        parser->flags.unbuffer = 1;
        STATE = DUMB_COLLECT;
}


int	dumb_nextstate(
        TEParser *parser, 
        int ch, 
        int *state)
{
        if ((ch == '\n') || (ch == '\r')) 
            *state = DUMB_NEWLINE;
        else if (PVARS.count < PINFO.max_buffer) 
            *state = DUMB_COLLECT;
        else *state = DUMB_MAXBUFF;
        return *state;
}

int	dumb_transition(
        TEParser *parser, 
        int newstate) 
{
        int oldstate = parser->state;
        
        switch (oldstate) {
        case DUMB_NEWLINE:
            switch (newstate) {
            case DUMB_COLLECT:
                tsMoveScreenCursor(parser->screen, 
                    ALA_SC(PAREA), PCROW+1);			/* fix me advance */
                PFIGN = 0;
                break;
            default:
                break;
            }
            break;
            
        case DUMB_MAXBUFF:
        case DUMB_COLLECT:
            switch (newstate) {
            case DUMB_NEWLINE:
                PFIGN = 1;
            case DUMB_MAXBUFF:
                dumb_unbuffer(parser);
            default:
                break;
            }
            break;
        }
        return newstate;
}

int	dumb_unbuffer(
        TEParser *parser)
{
        ALAttribU attrib;
        tsDataToScreen(parser->screen,  
            &attrib, PBUFF.data, PBUFF.count);
        PVARS.count = 0;
        teClearBuffer(&PBUFF);
        return 0;
}
