//
//  MTShell.h
//
//  Created by Herbert Ptzl on Mon Feb 04 2002.
//  Copyright (c) 2002 H.Ptzl. 
//
//  This file is part of MultiTerm.
//
//  MultiTerm is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  MultiTerm is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with MultiTerm; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#import <Foundation/Foundation.h>

#include "tstore.h"
#include "parser.h"

extern	NSString *MTShellDidChangeNotification;

@class	MTTask, MTView;

@interface MTShell : NSObject {
    @protected
    NSMutableDictionary *_dictionary;
    MTTask *_task;
    MTView *_view;
    TEParser *_parser;
    TSScreen _screen;
    TSStoreBuffer _store;
}

- (id)init;

/*

- (id)initWithLaunchPath:(NSString *)path;
- (id)initWithLaunchPath:(NSString *)path
        arguments:(NSArray *)arguments;
- (id)initWithLaunchPath:(NSString *)path
        arguments:(NSArray *)arguments
        environment:(NSDictionary *)dict;
*/

- (void)setScreenSize:(TVSize)aSize;

// - (void)setParser:(MTParser *)aParser;

- (void)updateBitmap:(TVBitMap *)bitmap 
    withOptions:(TVTextOpts *)options
    forViewport:(ALTextArea *)viewport 
    inRect:(NSRect)rect;


- (id)task;


@end

@interface MTShell (Adjustments)

- (void)adjustToSize:(TVSize)newsize;

@end

@interface MTShell (MTTaskFeatures)

- (void)setLaunchPath:(NSString *)path;
- (void)setArguments:(NSArray *)arguments;
- (void)setEnvironment:(NSDictionary *)dict;
- (void)setCurrentDirectoryPath:(NSString *)path;

- (NSString *)launchPath;
- (NSArray *)arguments;
- (NSDictionary *)environment;
- (NSString *)currentDirectoryPath;

- (void)launch;

- (id)standardInput;

- (void)interrupt;
- (void)terminate;

- (BOOL)suspend;
- (BOOL)resume;

- (int)processIdentifier; 
- (BOOL)isRunning;

- (int)terminationStatus;

@end
