//
//  MTShell.m
//
//  Created by Herbert Ptzl on Mon Feb 04 2002.
//  Copyright (c) 2002 H.Ptzl. 
//
//  This file is part of MultiTerm.
//
//  MultiTerm is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  MultiTerm is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with MultiTerm; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


#import "MTShell.h"

#import "MTTask.h"
#import "MTView.h"

#include "atextdef.h"
#include "tstoredef.h"
#include "tviewdef.h"


NSString *MTShellDidChangeNotification = 	@"ShellDidChangeNotification";	


@implementation MTShell

- (id)init {
    _dictionary = [[NSMutableDictionary alloc] init];
    _task = [[MTTask alloc] init];
    _parser = teNewParser(TEP_TYPE_DUMB, &_screen);
    _parser->flags.unbuffer = 1;
    _screen.flags.scroll = 1;
    _screen.flags.wrap = 1;
    
    _store.cindex = 0;
    _store.count = 0;
    _store.maxcount = 100;
    _store.offset = 0;
    _store.freerow = 0;
    _store.log2size = 7;
    _store.width = 80;
    _store.store = calloc(sizeof(TSStoreLine), 128);
    
    return self;
}

- (void)dealloc {
    if ([_task isRunning]) [_task kill];
    [_task release];
    [_dictionary release];
    [super dealloc];
}


- (void)setScreenSize:(TVSize)aSize {
    tsAdjustScreenSize(&_screen, aSize);
    /* todo: adjust store buffer & viewport */
    
    teSetParserScreen(_parser, &_screen);

    /* something is missing ? */
}


- (void)paintBitmap:(TVBitMap *)bitmap 
    withOptions:(TVTextOpts *)options
    inArea:(ALTextArea *)area {
    TVBitMap rowBitmap;
    int row;

    rowBitmap = *bitmap;
    alConfineTextArea(area, &_screen.area);
    for (row=ALA_SR(*area); row < ALA_ER(*area); row++) {
        ALTextRun run;
        rowBitmap.data = bitmap->data +
            TVO_CW(*options) * PIXSIZE * ALA_SC(*area) +
            TVO_CH(*options) * row * bitmap->rowstride; 
        run = _screen.run + TSS_AW(_screen)*row + ALA_SC(*area);
        tvBlitTextRun(options, &rowBitmap, run, ALA_AW(*area));
    }
}


- (void)paintBitmap:(TVBitMap *)bitmap
    withOptions:(TVTextOpts *)options
    forViewport:(ALTextArea *)viewport
    inArea:(ALTextArea *)area {
    // TVBitMap rowBitmap;
    // calc viewport fix me!
    [self paintBitmap:bitmap withOptions:options inArea:area];
}


- (void)updateBitmap:(TVBitMap *)bitmap 
    withOptions:(TVTextOpts *)options
    forViewport:(ALTextArea *)viewport 
    inRect:(NSRect)rect {
    
        // limit to rect
        // update necessary only
        // recognize view port changes 
        
    [self paintBitmap:bitmap
        withOptions:options
        forViewport:NULL
        inArea:&_screen.area];
        // fix me !
}


- (id)task {
    return _task;
}

@end


@implementation MTShell (Adjustments)

- (void)adjustToSize:(TVSize)newsize {
    if ((newsize.width != TSS_AW(_screen)) ||
        (newsize.height != TSS_AH(_screen))) {

        tsAdjustTerminalSize(&_store, &_screen, newsize);
        
//	  adjust view port? talk to view?
    }
}

@end


@implementation MTShell (MTTaskFeatures)


- (void)setLaunchPath:(NSString *)path {
    [_task setLaunchPath:path];
}

- (void)setArguments:(NSArray *)arguments {
    [_task setArguments:arguments];
}

- (void)setEnvironment:(NSDictionary *)dict {
    [_task setEnvironment:dict];
}

- (void)setCurrentDirectoryPath:(NSString *)path {
    [_task setCurrentDirectoryPath:path];
}


- (NSString *)launchPath {
    return [_task launchPath];
}

- (NSArray *)arguments {
    return [_task arguments];
}

- (NSDictionary *)environment {
    return [_task environment];
}

- (NSString *)currentDirectoryPath {
    return [_task currentDirectoryPath];
}


- (void)launch {
    [_task launch];
    [[NSNotificationCenter defaultCenter] 
        addObserver:self 
        selector:@selector(taskDataAvailable:) 
            name:NSFileHandleDataAvailableNotification
            object:[_task standardInput]];
    [[_task standardOutput] waitForDataInBackgroundAndNotify];

}

- (void)taskDataAvailable:(NSNotification *)note
{
    NSData *data;

    data = [[note object] availableData];
    if (data && [data length]) {
        NSString *string = [[NSString alloc] 
            initWithData:data encoding:NSASCIIStringEncoding];
        teParseInput(_parser, [string cString], [string length]);
        NSLog(@"note received ... %@", string);
        [[NSNotificationCenter defaultCenter]
            postNotificationName:MTShellDidChangeNotification 
            object:self];
        [string release];            
    }
    if (data && [data length])
        [[note object] waitForDataInBackgroundAndNotify];
}


- (id)standardInput {
    return [_task standardInput];
}


- (void)interrupt {
    [_task interrupt];
}

- (void)terminate {
    [_task terminate];
}

- (void)kill {
    [_task kill];
}


- (BOOL)suspend {
    return [_task suspend];
}

- (BOOL)resume {
    return [_task resume];
}


- (int)processIdentifier {
    return [_task processIdentifier];
}
 
- (BOOL)isRunning {
    return [_task isRunning];
}


- (int)terminationStatus {
    return [_task terminationStatus];
}


@end
