//
//  MTTask.m
//
//  Created by Herbert Ptzl on Mon Feb 04 2002.
//  Copyright (c) 2002 H.Ptzl. 
//
//  This file is part of MultiTerm.
//
//  MultiTerm is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  MultiTerm is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with MultiTerm; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


#include "ptyhelper.h"

#import "MTTask.h"

static	NSString *MTTaskLaunchPath = 		@"LaunchPath";	
static	NSString *MTTaskRealName = 		@"RealName";	
static	NSString *MTTaskArguments = 		@"Arguments";	
static	NSString *MTTaskEnvironment = 		@"Environment";	
static	NSString *MTTaskCurrentDirectoryPath = 	@"CurrentDirectoryPath";	

static	NSString *MTTaskStandardInput = 	@"StandardInput";	
static	NSString *MTTaskStandardOutput = 	@"StandardOutput";	

// 	static	NSString *MTTaskStandardError = 	@"StandardError";	


@implementation MTTask

- (id)init {
    _dictionary = [[NSMutableDictionary alloc] init];
    _hasExeced = NO;
    _isRunning = NO;
    _suspendCount = 0;
    _terminationStatus = 0;
    _pid = 0;
    return self;
}

- (void)dealloc {
    if (_isRunning) [self terminate];
    [_dictionary release];
    [super dealloc];
}


- (void)setLaunchPath:(NSString *)path {
    [_dictionary setObject:path forKey:MTTaskLaunchPath];
    [_dictionary setObject:path forKey:MTTaskRealName];
}

- (void)setArguments:(NSArray *)arguments {
    [_dictionary setObject:arguments forKey:MTTaskArguments];
}

- (void)setEnvironment:(NSDictionary *)dict {
    [_dictionary setObject:dict forKey:MTTaskEnvironment];
}

- (void)setCurrentDirectoryPath:(NSString *)path {
    [_dictionary setObject:path forKey:MTTaskCurrentDirectoryPath];
}


- (NSString *)launchPath {
    return [_dictionary objectForKey:MTTaskLaunchPath];
}

- (NSArray *)arguments {
    return [_dictionary objectForKey:MTTaskArguments];
}

- (NSDictionary *)environment {
    return [_dictionary objectForKey:MTTaskEnvironment];
}

- (NSString *)currentDirectoryPath {
    return [_dictionary objectForKey:MTTaskCurrentDirectoryPath];
}


- (void)launch {
    NSString *launchPath;
    NSString *currentDirectoryPath;
    NSArray *arguments;
    NSDictionary *environment;
    NSEnumerator *enumerator;

    char const ** avp;
    char const ** evp;
    const char *lp; 
    char *sp;
    int i, avc, evc;
    id key;
    
    arguments = [_dictionary objectForKey:MTTaskArguments];
    environment = [_dictionary objectForKey:MTTaskEnvironment];
    launchPath = [_dictionary objectForKey:MTTaskLaunchPath];
    currentDirectoryPath = [_dictionary objectForKey:MTTaskEnvironment];
    
    avc = [arguments count];
    evc = [environment count];
    avp = malloc(sizeof(char *const)*(avc+2));
    evp = malloc(sizeof(char *const)*(evc+1));
    
    for (i=0; i<avc; i++) {
        avp[i+1] = [[arguments objectAtIndex:i] cString];
    }
    avp[avc] = NULL;
    avp[0] = [[_dictionary objectForKey:MTTaskRealName] cString];

    enumerator = [environment keyEnumerator];
    i = 0;
    while ((key = [enumerator nextObject]) && (i < evc)) {
        NSString *str = [NSString stringWithFormat:@"%s=%s", 
            key, [environment objectForKey:key]];
        evp[i++] = [str cString];
    }
    evp[evc] = NULL;
    lp = [launchPath cString];
    
//    openMasterSlavePTY(_fileDesc, &mp, &sp);    
//    NSLog(@"master/slave = %s(%d),%s(%d)", mp, _fileDesc[0], sp, _fileDesc[1]);
    _pid = forkPTYSlave(_fileDesc, &sp, lp, avp, evp);
    NSLog(@"master/slave = %d,%s", _fileDesc[0], sp);
    NSLog(@"path=%s, avp[0]=%s", lp, avp[0]);
    if (_pid > 0) _isRunning = 1;
}

- (id)standardInput {
    NSFileHandle *handle;
    
    handle = [_dictionary objectForKey:MTTaskStandardInput];
    if (!handle) {
        handle = [[NSFileHandle alloc] 
            initWithFileDescriptor:_fileDesc[0]];
        [_dictionary setObject:handle forKey:MTTaskStandardInput];
        [_dictionary setObject:handle forKey:MTTaskStandardOutput];
    }
    return handle;
}

- (id)standardOutput {
    NSFileHandle *handle;
    
    handle = [_dictionary objectForKey:MTTaskStandardOutput];
    if (!handle) {
        handle = [[NSFileHandle alloc] 
            initWithFileDescriptor:_fileDesc[0]];
        [_dictionary setObject:handle forKey:MTTaskStandardInput];
        [_dictionary setObject:handle forKey:MTTaskStandardOutput];
    }
    return handle;
}

- (id)standardError {
    return nil;
}


- (void)interrupt {
    kill(_pid, SIGINT);
}

- (void)terminate {
    kill(_pid, SIGTERM);
}

- (void)kill {
    kill(_pid, SIGKILL);
}


- (BOOL)suspend {
    if (_suspendCount++) return NO;
    kill(_pid, SIGSTOP);
    return YES;
}

- (BOOL)resume {
    if (_suspendCount == 0) {
        kill(_pid, SIGCONT);
        return YES;
    } else _suspendCount--;
    return NO;
}


- (int)processIdentifier {
    return _pid;
}
 
- (BOOL)isRunning {
    return _isRunning;
}


- (int)terminationStatus {
    return _terminationStatus;
}




@end
