//
//  TermView.m
//
//  Created by Herbert Ptzl on Sun Feb 03 2002.
//  Copyright (c) 2002 H.Ptzl. 
//
//  This file is part of MultiTerm.
//
//  MultiTerm is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  MultiTerm is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with MultiTerm; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA


#import "MTView.h"
#import "MTShell.h"
#import "MTTask.h"

#import "Helper.h"

#include "genhelper.h"
#include "tviewdef.h"


@implementation MTView

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        _textopts = [Helper defaultTextOpts];
        [self adjustToFrame:frame];
    }
    return self;
}

- (void)setFrame:(NSRect)frameRect {
    [self adjustToFrame:frameRect];
    [super setFrame:frameRect];
}


- (void)drawRect:(NSRect)rect {
    // Drawing code here.

    [_shell updateBitmap:&_bitmap 
        withOptions:_textopts 
        forViewport:&_viewport 
        inRect:rect];
    
    NSDrawBitmap(_bounds, TVWidth(_bitmap), TVHeight(_bitmap), 
        8, 3, 32, _bitmap.rowstride, FALSE, FALSE, 
        NSDeviceRGBColorSpace, &_bitmap.data);    
    /* correct ? fix me */
}



- (void)setShell:(MTShell *)aShell {
    [aShell retain];
    [_shell release];
    
    [[NSNotificationCenter defaultCenter] 
        addObserver:self 
        selector:@selector(shellDidChange:) 
            name:MTShellDidChangeNotification
            object:aShell];

    _shell = aShell;
}

- (MTShell *)shell {
    return _shell;
}


- (NSSize)viewSizeForTextArea:(ALTextArea *)area {
    NSSize viewSize = {0, 0};
    return viewSize;		/* fix me */
}

- (ALTextArea *)textAreaForRect:(NSRect)aRect {
    return NULL;
}


- (TVBitMap *)bitmap {
    return &_bitmap;
}

- (ALTextArea)viewport {
    return _viewport;
}


@end

@implementation MTView (Adjustments)

- (void)adjustToFrame:(NSRect)frameRect {
    TVSize newsize;
    
    newsize.width = NSWidth(frameRect);
    newsize.height = NSHeight(frameRect);
    tvResizeBitMap(&_bitmap, newsize);
    
    newsize.width = INTRD(newsize.width/TVO_CW(*_textopts));
    newsize.height = INTRD(newsize.height/TVO_CH(*_textopts));
//    tvResizeViewPort(&_viewport, newsize);			/* fix me */
    
    [_shell adjustToSize:newsize];
}

@end

@implementation MTView (Events)

- (BOOL)acceptsFirstResponder {
    return YES;
}


- (void)doCommandBySelector:(SEL)aSelector {

}

- (void)insertText:(NSString *)aString {
    [(NSFileHandle *)[_shell standardInput] writeData:
        [aString dataUsingEncoding:NSASCIIStringEncoding]];
}

- (void)keyDown:(NSEvent *)theEvent {
    // You may want to do something here.
    [(NSFileHandle *)[_shell standardInput] writeData:
        [[theEvent characters]
            dataUsingEncoding:NSASCIIStringEncoding]];
    NSLog(@"key down %@", theEvent);

//    [self interpretKeyEvents:[NSArray arrayWithObject:theEvent]];
}

- (void)shellDidChange:(NSNotification *)note
{
    NSLog(@"shell did change ...");
    [self setNeedsDisplay:YES];
}


@end
