/*
 *  atext.c
 *
 *  Created by Herbert Poetzl on Mon Feb 04 2002.
 *  Copyright (c) 2002 H.Poetzl. 
 *
 *  This file is part of MultiTerm.
 *
 *  MultiTerm is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  MultiTerm is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MultiTerm; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "atext.h"
#include "atextdef.h"

#include <stdlib.h>




void	alConfineTextArea(
        ALTextArea *area,
        const ALTextArea *bound)
{
        if (ALA_SC(*area) < ALA_SC(*bound))
            ALA_SC(*area) = ALA_SC(*bound);
        if (ALA_SR(*area) < ALA_SR(*bound))
            ALA_SR(*area) = ALA_SR(*bound);
        if (ALA_EC(*area) > ALA_EC(*bound))
            ALA_EC(*area) = ALA_EC(*bound);
        if (ALA_ER(*area) > ALA_ER(*bound))
            ALA_ER(*area) = ALA_ER(*bound);
}        

void	alExtendTextArea(
        ALTextArea *area,
        const ALTextArea *extend)
{
        if (ALA_SC(*area) > ALA_SC(*extend))
            ALA_SC(*area) = ALA_SC(*extend);
        if (ALA_SR(*area) > ALA_SR(*extend))
            ALA_SR(*area) = ALA_SR(*extend);
        if (ALA_EC(*area) < ALA_EC(*extend))
            ALA_EC(*area) = ALA_EC(*extend);
        if (ALA_ER(*area) < ALA_ER(*extend))
            ALA_ER(*area) = ALA_ER(*extend);
}        




void	alSetSubTextArea(
        ALTextRun run, 
        ALTextArea *area,
        ALTextChar ch,
        int rowstride)
{
        ALTextRun dp;
        unsigned offset, height, width;
        int c,r;

        offset = ALA_SC(*area) + ALA_SR(*area) * rowstride;
        height = ALA_AH(*area);
        width = ALA_AW(*area);
        
        dp = run + offset;
        
        for (r=0; r<height; r++) {
            for (c=0; c<width; c++) {
                dp[c] = ch;
            }
            dp += rowstride;
        }
}


void	alSetTextArea(
        ALTextRun run, 
        ALTextArea *area,
        ALTextChar ch)
{
        ALTextRun dp;
        unsigned count;
        int n;

        count = ALA_AH(*area) * ALA_AW(*area);        
        dp = run;
        
        for (n=0; n<count; n++)
            dp[n] = ch;
}


void	alScrollSubTextArea(
        ALTextRun run, 
        ALTextArea *area,
        int rowstride,
        int rows)
{
        ALTextRun sp, dp;
        unsigned offset, count, width;
        int delta, n;

        offset = ALA_SC(*area) + ALA_SR(*area) * rowstride;
        count = ALA_AH(*area) - abs(rows);
        width = ALA_AW(*area);
        
        if (rows > 0) {
            delta = rowstride;
            dp = run + offset;
            sp = dp + rows * delta;
        } else {
            delta = -rowstride;
            offset -= count * delta;
            sp = run + offset;
            dp = sp - rows * delta;
        }
        
        for (n=0; n<count; n++) {
            memcpy(dp, sp, width * ALC_SIZE);
            sp += delta; dp += delta;
        }
}


void	alScrollTextArea(
        ALTextRun run, 
        ALTextArea *area,
        int rows)
{
        ALTextRun sp, dp;
        unsigned count, width;

        count = ALA_AH(*area) - abs(rows);
        width = ALA_AW(*area);
        
        if (rows > 0) {
            dp = run;
            sp = dp + rows * width;
        } else {
            sp = run;
            dp = sp - rows * width;
        }
        memcpy(dp, sp, count * width * ALC_SIZE);
}

/* 	should this help remapping the characters? */


int	alAllocTextRun(
        ALTextRun *run,
        int count)
{
        if (run) {
            ALTextRun new;
            new = malloc(count * ALC_SIZE);
            *run = new;
        }
        return 0;
}

int	alReallocTextRun(
        ALTextRun *run,
        int count)
{
        if (run) {
            ALTextRun new;
            new = realloc(*run, count * ALC_SIZE);
            *run = new;
        }
        return 0;
}

int	alFreeTextRun(
        ALTextRun *run)
{
        if (run && (*run)) {
            free(*run);
            *run = NULL;
        }
        return 0;
}


int	alResizeTextRun(
        ALTextRun *run,
        ALTextArea *oldarea,
        ALTextArea *area)
{
        if (run) {
            if (*run) 
                alReallocTextRun(run, ALA_SIZE(*area));
            else alAllocTextRun(run, ALA_SIZE(*area));
        }
        return 0;
}



int	alIndexOfTextChar(
        ALTextRun run,
        ALTextChar ch,
        int count)
{
        unsigned index;
        
        for (index=0; index<count; index++) {
            if (run[index].value.ascii == ch.value.ascii)
                break;
        }
        return index;
}



