/*
 *  atext.h
 *
 *  Created by Herbert Poetzl on Mon Feb 04 2002.
 *  Copyright (c) 2002 H.Poetzl. 
 *
 *  This file is part of MultiTerm.
 *
 *  MultiTerm is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  MultiTerm is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MultiTerm; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef	_ATEXT_INCLUDE_
#define	_ATEXT_INCLUDE_


typedef	struct 	_al_color 		ALColor;
typedef	struct 	_al_attributes 		ALAttrib;
typedef union 	_al_attrib_union	ALAttribU;

typedef union 	_al_char_union 		ALCharU;
typedef	struct 	_al_text_char 		ALTextChar;
typedef	struct 	_al_text_char *		ALTextRun;

typedef struct 	_al_text_coordinate 	ALTextCoor;
typedef	struct 	_al_text_area 		ALTextArea;
typedef struct	_al_text_line 		ALTextLine;


struct 	_al_color {
            unsigned back : 4;
            unsigned text : 4;
        } __attribute__ ((packed));

struct	_al_attributes {
        ALColor color;
        unsigned underline : 1;
        unsigned reverse : 1;
        unsigned blink : 1;
        unsigned bold : 1;
        unsigned invisible : 1;
        unsigned protect : 1;
        unsigned charset : 2;
}	__attribute__ ((packed));

union	_al_attrib_union {
        ALAttrib detail;
        unsigned short all; 
} 	__attribute__ ((packed));


union 	_al_char_union {
        unsigned short uni;
        unsigned char ascii;
}	__attribute__ ((packed));

struct	_al_text_char {
        ALCharU value;
        ALAttribU attrib;
}	__attribute__ ((packed));



struct	_al_text_coordinate {
        signed int col;
        signed int row;
}	__attribute__ ((packed)); 

struct	_al_text_area {
        ALTextCoor topleft;
        ALTextCoor botright;
}	__attribute__ ((packed));

struct	_al_text_line {
        unsigned int offset;
        unsigned short count;
        ALTextRun run;
}	__attribute__ ((packed));



/*	atext procedures	*/

void	alConfineTextArea(
        ALTextArea *area,
        const ALTextArea *bound);

void	alExtendTextArea(
        ALTextArea *area,
        const ALTextArea *extend);


void	alSetSubTextArea(
        ALTextRun run, 
        ALTextArea *area,
        ALTextChar ch,
        int rowstride);

void	alSetTextArea(
        ALTextRun run, 
        ALTextArea *area,
        ALTextChar ch);


void	alScrollSubTextArea(
        ALTextRun run, 
        ALTextArea *area,
        int rowstride,
        int rows);

void	alScrollTextArea(
        ALTextRun run, 
        ALTextArea *area,
        int rows);


int	alAllocTextRun(
        ALTextRun *run,
        int count);

int	alReallocTextRun(
        ALTextRun *run,
        int count);

int	alFreeTextRun(
        ALTextRun *run);


int	alResizeTextRun(
        ALTextRun *run,
        ALTextArea *oldarea,
        ALTextArea *area);



int	alIndexOfTextChar(
        ALTextRun run,
        ALTextChar ch,
        int count);
        

#endif 	/* _ATEXT_INCLUDE_ */

