/*
 *  parser.c
 *
 *  Created by Herbert Poetzl on Tue Feb 05 2002.
 *  Copyright (c) 2002 H.Poetzl. 
 *
 *  This file is part of MultiTerm.
 *
 *  MultiTerm is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  MultiTerm is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MultiTerm; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "parser.h"
#include "parserdef.h"

#include "DUMBparse.h"
#include "VTparse.h"
#include "TEKparse.h"

#include <stdlib.h>



void	teAllocBuffer(
        TEBuffer *buffer, 
        int size) {
        int l2s;
        
        for (l2s=4; l2s<32; l2s++) 
            if ((1L << l2s) >= size) break;
        buffer->log2size = l2s;
        buffer->data = malloc(1L << l2s);
        buffer->count = 0;
}

void	teResetBuffer(
        TEBuffer *buffer) {
        buffer->count = 0;
}

void	teClearBuffer(
        TEBuffer *buffer) {
        buffer->count = 0;
}

void	teAddToBuffer(
        TEBuffer *buffer,
        char ch) {
        unsigned int count;
        
        count = buffer->count;
        if (count == (1L << buffer->log2size)) {
            buffer->log2size++;
            buffer->data = realloc(buffer->data, 
                (1L <<  buffer->log2size));
        }
        buffer->data[count] = ch;
        buffer->count = (count + 1);
}


TEParser * teNewParser(
        TEParserType type,
        TSScreen *screen)
{        
        TEParser *new;
        int buffersize;
        
        new = calloc(sizeof(TEParser), 1);
        new->type = type;
        teSetParserScreen(new, screen);
        
        switch (type) {
        case TEP_TYPE_DUMB:
            buffersize = dumb_init(new);
            teAllocBuffer(&new->buffer, buffersize);
            break;
            
        case TEP_TYPE_VT:
            break;
            
        case TEP_TYPE_TEK:
            break;
            
        default:
            break;
        }
        return new;
}

void	teSetParserScreen(
        TEParser *parser, 
        TSScreen *screen)
{
        parser->screen = screen;
        /* recalc whatever ??? */
}

void	teResetParser(
        TEParser *parser, 
        int level) 
{
        parser->reset(parser);
        teResetBuffer(&parser->buffer);        
}

int	teParseInput(
        TEParser *parser, 
        const char *input,
        unsigned length) {
        
        int ret, newstate, state;
        int ch;
        
        while (length--) {
            state = parser->state;
            ch = *input++; newstate = state;
            ret = parser->nextstate(parser, ch, &newstate);
            if ((ret > 0) && (newstate != state)) {
                ret = parser->transition(parser, newstate);
                parser->state = newstate;
            }
            if (!parser->flags.ignore)
                teAddToBuffer(&parser->buffer, ch);
        }
        if (parser->flags.unbuffer) {
            parser->unbuffer(parser);
        }
        return 0;
}




