/*
 *  parser.h
 *
 *  Created by Herbert Poetzl on Tue Feb 05 2002.
 *  Copyright (c) 2002 H.Poetzl. 
 *
 *  This file is part of MultiTerm.
 *
 *  MultiTerm is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  MultiTerm is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MultiTerm; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef	_PARSER_INCLUDE_
#define	_PARSER_INCLUDE_

#define	_ONLY_PARSER_STRUCT_

#include "DUMBparse.h"
#include "VTparse.h"
#include "TEKparse.h"

#undef	_ONLY_PARSER_STRUCT_

#include "tstore.h"


enum 	_te_parser_type {
        TEP_TYPE_DUMB = 1,
        TEP_TYPE_VT,
        TEP_TYPE_TEK,
};

typedef	enum 	_te_parser_type 	TEParserType;

typedef	struct 	_te_parser_buffer 	TEBuffer;
typedef	struct 	_te_parser 		TEParser;


struct	_te_parser_buffer {
        char *data;
        int log2size;
        int count;
};

struct	_te_parser
{
        int type;
        int state;
        TEBuffer buffer;
        TSScreen *screen;
        
        int (*nextstate)(struct _te_parser *, int, int *);
        int (*transition)(struct _te_parser *, int);
        int (*unbuffer)(struct _te_parser *);
        void (*reset)(struct _te_parser *);
        union {
            struct _te_dumb_info pass;
            struct _te_vt_info vt;
            struct _te_tek_info tek;
        } info;
        union {
            struct _te_dumb_vars pass;
            struct _te_vt_vars vt;
            struct _te_tek_vars tek;
        } vars;
        struct {
            unsigned unbuffer:1;
            unsigned ignore:1;
        } flags;
};





TEParser * teNewParser(
        TEParserType type,
        TSScreen *screen);

void	teSetParserScreen(
        TEParser *parser, 
        TSScreen *screen);

void	teResetParser(
        TEParser *parser, 
        int level);

int	teParseInput(
        TEParser *parser, 
        const char *input,
        unsigned length);


#endif 	/* _PARSER_INCLUDE_ */

