/*
 *  ptyhelper.c
 *
 *  Created by Herbert Poetzl on Fri Jan 25 2002.
 *  Copyright (c) 2002 H.Poetzl. 
 *
 *  This file is part of MultiTerm.
 *
 *  MultiTerm is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  MultiTerm is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MultiTerm; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>

#include <sys/stat.h>
#include <sys/ioctl.h>

#include "ptyhelper.h"


#include <sys/types.h>
#include <grp.h>


#define	TTY_GROUP(gid)					\
        {						\
            struct group *grptr;			\
            if ( (grptr = getgrnam("tty")) != NULL)	\
                gid = grptr->gr_gid;			\
            else					\
                gid = -1;  /* not in group file */	\
        }						\


const char ptym[] = "/dev/pty";
const char ptys[] = "/dev/tty";

#if 0	/* man page definition */

const char pty1[] = "pqrsPQRS";
const char pty2[] = "0123456789abcdefghijklmnopqrstuvwxyz";

#else	/* default darwin? */

const char pty1[] = "pq";
const char pty2[] = "0123456789abcdef";

#endif


#define	PTYM_LEN	(sizeof(ptym)-1 + 2)
#define PTYS_LEN	(sizeof(ptys)-1 + 2)

#define	FORK	vfork



static 	inline
int	stringCopy(
        char **ptr, 
        const char *string,
        int len)
{
        if (ptr && string) {
            if (!len) len = strlen(string)+1;
            *ptr = malloc(len);
            if (*ptr) strncpy(*ptr, string, len);
            else return -1;
            return len;
        }
        return 0;
}


int	getFreePTY(
        char *master, 
        char *slave) 
{
        int i1, i2, mi, si, fd;
//        gid_t ttygid;
        
        mi = PTYM_LEN - 2;
        strncpy(master, ptym, mi);
        master[mi+2] = '\0';
        si = PTYS_LEN - 2;
        strncpy(slave, ptys, si);
        slave[si+2] = '\0';
         
        for (i1=0; i1<sizeof(pty1) - 1; i1++) {
            master[mi]=pty1[i1];
            slave[mi]=pty1[i1];
            for (i2=0; i2<sizeof(pty2) - 1; i2++) {
                master[mi+1]=pty2[i2];
                slave[mi+1]=pty2[i2];
                
                if (access(master, W_OK)) continue;
                fd = open(master, O_RDWR /* | O_NONBLOCK */);
                if (fd != -1) {
                    return fd;
                }
            }
        }        
        return -1;
}


int	openMasterSlavePTY(
        int fd[2],
        char **masterPath, 
        char **slavePath) 
{
        int mfd, sfd, gid;
        char master[PTYM_LEN+1];
        char slave[PTYS_LEN+1];

        if (!fd) return -1;
        mfd = getFreePTY(master, slave);
        if (mfd > 0) {
            TTY_GROUP(gid);

            /* following two functions don't work unless we're root */
            chown(slave, getuid(), gid);
            chmod(slave, S_IRUSR | S_IWUSR | S_IWGRP);

            sfd = open(slave, O_RDWR);
            if (sfd >= 0) {
                fd[0] = mfd;
                fd[1] = sfd;
                stringCopy(masterPath, master, PTYM_LEN+1);
                stringCopy(slavePath, slave, PTYM_LEN+1);
            } else {
                close(mfd);
                return -1;
            }
        }
        return 0;
}


int	forkPTYSlave(
        int fd[2],
        char *slavePath,
        const char *cmd,
        char *const argv[],
        char *const envp[]) 
{
        pid_t	pid;
        
        pid = FORK();
        if (pid == 0) {						/* child */
            
            if (ioctl(fd[1], TIOCSCTTY, (char *) 0) < 0)
                perror("TIOCSCTTY error");
                
            /* 	set slave's termios and window size
            if (slave_termios != NULL) {
                    if (tcsetattr(fds, TCSANOW, slave_termios) < 0)
                            err_sys("tcsetattr error on slave pty");
            }
            if (slave_winsize != NULL) {
                    if (ioctl(fds, TIOCSWINSZ, slave_winsize) < 0)
                            err_sys("TIOCSWINSZ error on slave pty");
            }
            */
            close(0); dup(fd[1]);				/* replace stdin */
            close(1); dup(fd[1]);				/* replace stdout */
            close(2); dup(fd[1]);				/* replace stderr */
            close(fd[0]);
            close(fd[1]);
            
            fprintf(stderr, "stderr test output\n");
            write(2, "stderr test output 2\n", 21);
            execve (cmd, argv, envp);
            exit(-1);		
            
        } else if (pid > 0) {					/* parent */
            return pid;
            
        } else {						/* failure */
            perror("fork():");
            return -1;
        }
}

