/*
 *  tstore.c
 *
 *  Created by Herbert Poetzl on Mon Feb 04 2002.
 *  Copyright (c) 2002 H.Poetzl. 
 *
 *  This file is part of MultiTerm.
 *
 *  MultiTerm is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  MultiTerm is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MultiTerm; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "tstore.h"
#include "tstoredef.h"

#include "atext.h"
#include "atextdef.h"

#include "tview.h"
#include "tviewdef.h"

#include "genhelper.h"

#include <stdlib.h>
#include <assert.h>
#include <math.h>



int	tsLorSToScreen(
        TSScreen *screen,
        ALTextLine *line,
        ALAttribU *attrib,
        const char *data,
        unsigned len)
{
        ALTextRun run, drun;
        ALTextArea linearea;
        int count, offset;
        int done, cols;

        if (line) {							/* working for tsLine... 	*/
            run = line->run;
            count = line->count;
        } else {							/* working for tsData... 	*/
            count = len;
        }
        done = 0;
        while (done < count) {
            linearea.topleft = screen->cursor;				/* cursor pos as start 		*/
            ALA_ER(linearea) = ALA_SR(linearea) + 1;			/* line = one row 		*/
            ALA_EC(linearea) = ALA_SC(linearea) + 			/* calc oversize? line 		*/
                (count - done);						
            SACONF(&linearea);						/* clip to screen 		*/
            SDIRTY(&linearea);						/* mark area dirty		*/
            cols = ALA_AW(linearea);					/* get num chars 		*/
            offset = ALA_C2O(screen->area, linearea.topleft);		/* calculate offset		*/
            drun = screen->run + offset;
            
            if (line) {							/* working for tsLine... 	*/
                memcpy(drun, run + done, cols * ALC_SIZE);		/* copy alchar run	 	*/
            } else {							/* working for tsData... 	*/
                int i;
                for (i=0; i<cols; i++) {
                    drun[i].value.ascii = data[done + i];		/* set ascii character		*/
                    drun[i].attrib = *attrib;				/* set attributes 		*/
                }
            }
        
            done += cols; SCURC += cols;				/* update position/done		*/
            if (count == done) break;					/* if done exit now ?		*/
            
            if (SWRAP) {						/* wrap enabled 		*/
                SCURC = TSS_SC(*screen);				/* move cursor to line start	*/
                if (SCURR == TSS_ER(*screen) - 1) {			/* last row of screen ?		*/
                    if (SSCRL) {
                        alScrollTextArea(screen->run,			/* scroll screen one line up	*/
                            &screen->area, 1); 
                        alSetSubTextArea(screen->run,			/* clear last screen line	*/
                            &SLLAR, SNULC, TSS_AW(*screen));				
                        SDIRTY(&screen->area);				/* set all dirty?, fix me 	*/
                    } else break;
                } else {						/* mid of screen ! 		*/
                    SCURR++;						/* move one row down		*/
                }
            } else break;						/* no wrap, work done, break	*/
        }
        return done;							/* return # visible chars	*/
}


int	tsLineToScreen(
        TSScreen *screen,
        ALTextLine *line)
{
        return tsLorSToScreen(screen, line, 
            0, 0, 0);
}


int	tsDataToScreen(
        TSScreen *screen,
        ALAttribU *attrib,
        const char *data,
        unsigned len)
{
        return tsLorSToScreen(screen, 0, 
            attrib, data, len);
}        

int	tsStringToScreen(
        TSScreen *screen,
        ALAttribU *attrib,
        const char *string)
{
        return tsLorSToScreen(screen, 0, 
            attrib, string, strlen(string));
}        

void	tsMoveScreenCursor(
        TSScreen *screen,
        int col,
        int row)
{
        int srows;
        
        if (row >= TSS_ER(*screen))
            TSS_CR(*screen) = TSS_ER(*screen) - 1;
        else if (row < TSS_SR(*screen))
            TSS_CR(*screen) = TSS_SR(*screen);
        else TSS_CR(*screen) = row;
        srows = row - TSS_CR(*screen);        

        TSS_CC(*screen) = MIN(MAX(col, 
            TSS_SC(*screen)), TSS_EC(*screen));
        
        if ((srows) && SSCRL) {
            alScrollTextArea(screen->run,
                &screen->area, srows);
            alSetSubTextArea(screen->run,				/* clear last screen line	*/
                &SLLAR, SNULC, TSS_AW(*screen));				
            SDIRTY(&screen->area);
        }
}        


        /* Bitstream Helper */

static	inline
void	putBits(
        BitSPtr *sptr,
        unsigned int value,
        unsigned bits)
{
        unsigned int *ptr;
        unsigned int bitpos;
        
        ptr = sptr->ptr;
        bitpos = sptr->bitpos;
        
        if ((bits + bitpos) > 32) {
            unsigned int stored = (32 - bitpos);
            unsigned int mask = (~0 << bitpos);
            *ptr++ = (*ptr & ~mask) | (value << bitpos);
            *ptr = value >> stored; 
            sptr->bitpos = bits - stored;
            sptr->ptr = ptr;
        } else {
            unsigned int mask = (~0 << bitpos);
            *ptr = (*ptr & ~mask) | (value << bitpos);
            sptr->bitpos = bitpos + bits;
        }
}

static	inline
void	putBit(
        BitSPtr *sptr,
        char value)
{
        unsigned int *ptr;
        unsigned int bitpos, bitmask;
        
        ptr = sptr->ptr;
        bitpos = sptr->bitpos;
        bitmask = (1 << bitpos);
        
        *ptr &= ~bitmask;
        if (value) *ptr |= bitmask;
        
        if (bitpos == 31) {
            sptr->bitpos = 0;
            sptr->ptr++;
        } else {
            sptr->bitpos++;
        }
}

static	inline
unsigned int getBits(
        BitSPtr *sptr,
        unsigned bits)
{
        unsigned int *ptr;
        unsigned int bitpos, value, mask;
        
        ptr = sptr->ptr;
        bitpos = sptr->bitpos;
        mask = (~0 >> (32 - bits));
        value = 0;
        
        if ((bits + bitpos) > 32) {
            unsigned int avail = (32 - bitpos);
            value = *ptr++ >> bitpos;
            value = (value | (*ptr << avail)) & mask; 
            sptr->bitpos = bits - avail;
            sptr->ptr = ptr;
        } else {
            value = (*ptr >> bitpos) & mask;
            sptr->bitpos = bitpos + bits;
        }
        return value;
}

static	inline
char	getBit(
        BitSPtr *sptr)
{
        unsigned int *ptr;
        unsigned int bitpos;
        char value;
        
        ptr = sptr->ptr;
        bitpos = sptr->bitpos;
        
        value = (*ptr >> bitpos) & 1;        
        if (bitpos == 31) {
            sptr->bitpos = 0;
            sptr->ptr++;
        } else {
            sptr->bitpos++;
        }
        return value;
}


        /* basic text store encoder (copy) */

void	tsEncodeTextStore(
        ALTextLine *line,
        TSStoreLine *store)
{
        unsigned int estimate, real;
        
        estimate = line->count * ALC_SIZE;
        if (store->data) free(store->data);
        store->data = malloc(estimate);
        
        real = estimate;
        memcpy(store->data, line->run, real);
        
        if (real < estimate) 
            store->data = realloc(store->data, real);
        store->count = line->count;
}

        /* basic text store decoder (copy) */

void	tsDecodeTextStore(
        TSStoreLine *store,
        ALTextLine *line,
        int skip)
{
        unsigned int len;
        
        len = MIN(store->count, line->count) - skip;
        if (len > 0) {
            memcpy(line->run, store->data + skip, 
                len * ALC_SIZE);
        } else len = 0;
        line->count = len;
}


void	tsUnpackStoreToTextRun(
        TSStoreBuffer *store, 
        ALTextRun *run,
        ALTextArea *area,
        int offset)
{
        assert(0);
}


void	tsUnpackStoreToScreen(
        TSStoreBuffer *store, 
        TSScreen *screen,
        ALTextArea *area)
{
        unsigned int index, offset, length;
        unsigned int width = TSS_AW(*screen);
        ALTextLine tl;
        ALTextArea rol;
        int skip, firstrow = 1;
        
        ALA_SET(rol, 0, 0, width, 1);					/* rest of line helper		*/

        tl.run = screen->run + (ALA_SR(*area) * width);			/* first row to update		*/
        length = ALA_AH(*area) * width;					/* length of text run ...	*/

        offset = screen->offset + ALA_SR(*area);
        index = tsStoreIndexForOffset(store, offset);			/* get corresp. line index	*/

        while ((index >= 0) && (length > 0) &&				/* store and space available?	*/
            (offset < store->freerow)) {				/* store lines available?	*/
            if (firstrow) {
                skip = width * 						/* calculate data skip		*/
                    (offset - TSB_LO(*store, index));		
                firstrow = 0;
            } else skip = 0;
            tl.count = length;						/* max length of text line	*/
            tsDecodeTextStore(&store->store[index], &tl, skip);		/* unpack stored line data	*/
            
            if (length > tl.count) {					/* some area left?		*/
                ALA_SC(rol) = tl.count % width;				/* rest of line position	*/
                alSetTextArea(tl.run + tl.count, &rol, SNULC);		/* clear rest of line		*/
                tl.count += ALA_AW(rol);				/* update count (full line)	*/
            }
            tl.run += tl.count;						/* advance rows			*/
            length -= tl.count;						/* reduce left over area	*/
            
            offset += tl.count / width;					/* advance offset		*/
            index = CADD(index, 1, store->maxcount);			/* advance index		*/
        }
        
        if (length > 0) {						/* some unused area left?	*/
            ALA_SC(rol) = 0;
            ALA_ER(rol) = length / width;
            alSetTextArea(tl.run, &rol, SNULC);				/* clear rest of screen		*/
        }
}


void	tsUpdateStoreFromScreen(
        TSStoreBuffer *store, 
        TSScreen *screen,
        ALTextArea *area,
        int *cursorindex)
{
        unsigned int row, rows;
        unsigned int width = TSS_AW(*screen);
        ALTextRun run, helper;
        ALTextLine tl;
        int firstrow = 1;
        
        if (cursorindex) *cursorindex = -1;				/* default no cursor		*/
        helper = 0;
        
        row = ALA_SR(*area);						/* first row to update		*/
        run = screen->run + (row * width);				/* run for first row 		*/
            
        while (row < ALA_ER(*area)) {					/* do we have some rows left?	*/
            unsigned int length, offset;
            unsigned int reuse = 0;
            int index;
            
            offset = SLOFF + row;					/* screen offset + row		*/
            length = alIndexOfTextChar(run, SNULC, 			/* get line (rest) length	*/
                (ALA_ER(*area) - row) * width);
                
            if (firstrow) {						/* first row -> partial line	*/
                index = tsStoreIndexForOffset(store, offset);		/* get corresp. line index	*/
                if (index >= 0) {					/* line in store *G*		*/
                    tl.offset = TSB_LO(*store, index);			/* store line offset		*/
                    reuse = offset - tl.offset;				/* calc reused store rows	*/
                }
                if (reuse) {						/* do we reuse rows?		*/
                    unsigned int total;
                    tl.count = reuse * width;				/* reused data amount		*/
                    total = length + tl.count;				/* new total line length	*/
                    alAllocTextRun(&helper, total);			/* allocate first line helper	*/
                    tl.run = helper;
                    tsDecodeTextStore(&store->store[index], &tl, 0);	/* reuse stored line data	*/
                    memcpy(tl.run + (reuse * width), 			/* copy new line data		*/
                        run, length * ALC_SIZE);			
                    tl.count = total;					/* save total line length	*/
                }
                if (index >= 0) tsFreeStoreLines(store, index);		/* free overlapping lines	*/
                firstrow = 0;						/* only one first row *G*	*/
            } 
            if (!helper) {						/* full line run 		*/
                tl.offset = offset;					/* absolute line offset		*/
                tl.count = length;					/* text line length		*/
                tl.run = run;						/* text line start		*/
            }
            
            index = tsReserveStoreLines(store, 1);			/* reserve (new) store line	*/
            tsEncodeTextStore(&tl, &store->store[index]);		/* encode store line		*/
            
            if (helper) alFreeTextRun(&helper);				/* free first line helper run 	*/

            rows = (tl.count / width) + 1;				/* rows added to store		*/
            store->freerow += (rows - 1);				/* adjust free row		*/
            if ((TSS_CR(*screen) >= row) && 				/* cursor within rows		*/
                (TSS_CR(*screen) < (row + rows)))				
                if (cursorindex) *cursorindex = index;			/* save cursor index		*/
            row += rows - reuse;					/* add total rows stored	*/
            run += (rows - reuse) * width;				/* advance rows			*/
        }
}

int	tsReserveStoreLines(
        TSStoreBuffer *store,
        int count)
{
        unsigned int index, first, width;
        unsigned int maxcount, cindex;

        maxcount = store->maxcount;					/* local copy			*/
        cindex = store->cindex;						/* local copy			*/
        first = CADD(cindex, store->count, maxcount);			/* calc circular index		*/
        width = store->width;						/* local copy			*/
        index = first;							/* start at first		*/
        
        while (count--) {
            TSStoreLine *slp = &store->store[index];			/* store line for index		*/
            
            if (store->count == maxcount) {				/* drop first line ...		*/
                store->offset += 1 + 
                    TSB_LC(*store, (cindex % maxcount)) / width;	/* add first line rows		*/
                cindex++;
            } else {							/* append new line ...		*/
                store->count++;
            }

            if (slp->data) {						/* does line contain data?	*/
                free(slp->data);					/* free line data		*/
                slp->data = 0;
            }
            slp->count = 0;						/* set empty line ...		*/
            slp->offset = store->freerow++;				/* calc offset for new line	*/
            index = CADD(index, 1, maxcount);				/* next circular index		*/
        }
        return first;
}

                        
                        
int	tsFreeStoreLines(
        TSStoreBuffer *store,
        int index)
{
        unsigned int bound, lines;
        unsigned int maxcount, cindex;

        maxcount = store->maxcount;					/* local copy			*/
        cindex = store->cindex;						/* local copy			*/
        lines = 0;
        
        assert(CIRA(index, cindex, store->count, maxcount));		/* assert within store!		*/

        if (store->count) {
            bound = CADD(cindex, store->count, maxcount);		/* calc circular bound		*/
            
            store->freerow = store->store[index].offset;		/* correct last row in store	*/
            while (index != bound) {					/* while within bound		*/
                TSStoreLine *slp = &store->store[index];
                if (slp->data) {
                    free(slp->data);					/* free store line data		*/
                    slp->data = 0;		
                    slp->count = 0;
                }
                lines++;
                index = CADD(index, 1, maxcount);			/* next store index		*/
            }
            store->count -= lines;					/* reduce store count		*/
        }
        return lines;
}

int	tsLineForOffset(
        TSStoreBuffer *store,
        int offset)
{
        float delta, posa, posb;
        unsigned int line, width, count;
        unsigned int maxcount, cindex;

        if ((offset >= store->offset) &&				/* line within store limits?	*/
            (offset < store->freerow)) {
            
            width = store->width;					/* local copy			*/
            maxcount = store->maxcount;					/* local copy			*/
            count = store->count;					/* local copy			*/
            cindex = (store->cindex % maxcount);			/* wrap to circular buffer	*/
            posa = 0.0; posb = count - 1.0;				/* full search range		*/
            
            do {
                unsigned int index, slo;
    
                delta = (posb - posa) / 2;				/* max distance to move		*/
                line = (posb + posa) / 2;				/* check in the middle		*/
                index = CADD(line, cindex, maxcount);			/* wrap to circular buffer	*/
                slo = TSB_LO(*store, index);				/* store line offset		*/
                if (slo > offset)					/* line offset too high?	*/
                    posb = ceil(posb - delta);				/* lower half valid		*/
                else posa = floor(posa + delta);			/* upper half valid		*/
            } while (delta > 0.5);					/* stop at final value?		*/
            return (store->cindex + line);				/* add circular buffer index	*/
        } else {
            return -1;							/* not in store *G*		*/
        }
}

int	tsStoreIndexForLine(
        TSStoreBuffer *store,
        int line)
{
        int index;
        
        if ((line >= store->cindex) && 					/* is line in store?		*/
            (line < store->freerow)) {					
            index = CSUB(line, store->cindex, store->maxcount);		/* wrap circular buffer		*/
            return index;
        }
        else return -1;
}

int	tsStoreIndexForOffset(
        TSStoreBuffer *store,
        int offset)
{
        int line, index;
        
        line = tsLineForOffset(store, offset);				/* get line for offset		*/
        if (line >= 0) {						/* is line in store?		*/
            assert((line >= store->cindex) && 				/* line in store range?		*/
                (line < (store->cindex + store->count)));		
            index = CSUB(line, store->cindex, store->maxcount);		/* wrap circular buffer		*/
            return index;
        }
        else return -1;							/* line not in store		*/
}


void	tsRecalcStoreBuffer(
        TSStoreBuffer *store,
        int width)
{
        unsigned int lines, bound;
        unsigned int maxcount, index, offset;
        
        if (store->count) {						/* anything to do?		*/
            lines = 0;							/* local copy			*/
            maxcount = store->maxcount;					/* local copy			*/
            offset = store->offset;					/* local copy			*/
            index = (store->cindex % maxcount);				/* store line index		*/
            bound = CADD(index, store->count, maxcount);		/* calc index bound 		*/
            do {
                TSStoreLine *slp = &store->store[index];
                slp->offset = offset;					/* set line offset		*/
                offset += (slp->count / width) + 1;			/* add line count		*/
                index = CADD(index, 1, maxcount);			/* next line index		*/
            } while (index != bound);					/* boundary hit?		*/
            store->freerow = offset;					/* set first free row		*/
            store->width = width;					/* set calc screen width	*/
        }
}


void	tsAdjustScreenSize(
        TSScreen *screen,
        TVSize newsize)
{
        ALTextArea newarea;
        
        ALA_SC(newarea) = 0;	
        ALA_SR(newarea) = 0;
        ALA_EC(newarea) = newsize.width;	
        ALA_ER(newarea) = newsize.height;

        alResizeTextRun(&screen->run,					/* resize screen text run	*/
            &screen->area, &newarea);
        alResizeTextRun(&screen->buffer,				/* resize store buffer run	*/
            &screen->area, &newarea);
        screen->area = newarea;						/* setup screen area ... 	*/
        
        SLLAR = newarea;						/* setup last line hint ... 	*/
        ALA_SR(SLLAR) = ALA_ER(SLLAR)-1;
        
        SNULC.value.ascii = 0;	/* does this belong here? */
}


void   	tsSmartFocusCursor(
        TSStoreBuffer *store,
        TSScreen *screen,
        int rowpan)
{
        unsigned int crow;
        unsigned int height;

        height = TSS_AH(*screen);					/* local copy			*/
        crow = TSS_CR(*screen) + screen->offset;			/* absolute cursor row		*/

        if (screen->offset > rowpan)
            screen->offset -= rowpan;					/* pan screen vertically	*/
        else screen->offset = 0;
        
        if (crow >= (screen->offset + height)) {			/* cursor below screen		*/
            screen->offset = crow - (height - 1);			/* move screen down		*/
        } else if (crow < screen->offset) {
            screen->offset = crow;					/* move screen up 		*/
        }
        TSS_CR(*screen) = crow - screen->offset;			/* set new cursor row		*/
}

#include <stdio.h>

void	_dump_store(
        TSStoreBuffer *store)
{
        unsigned int index, count;

        count = store->count;
        index = store->cindex % store->maxcount;
        printf("STORE %p [%5d/%5d] CI=%5d\n", 
            store, count, store->maxcount, store->cindex);
        while (count--) {
            int i;
            printf("[%5d] (%5d,%5d) >", 
                index, store->store[index].offset, 
                store->store[index].count);
            for (i=0; i<store->store[index].count; i++) {
                ALTextChar ch;
                ch = *(ALTextChar *)&store->store[index].data[i];
                printf("%02X ", ch.value.ascii);
            }
            printf("\n");
            index = CADD(index, 1, store->maxcount);
        }
}

void	tsAdjustTerminalSize(
        TSStoreBuffer *store,
        TSScreen *screen, 
        TVSize newsize)
{
        int cursorindex, rowdelta; 
        unsigned int linebase, virtcol;
                
        tsUpdateStoreFromScreen(store, screen, 				/* copy screen to store		*/
            &screen->area, &cursorindex);

        linebase = TSB_LO(*store, cursorindex);				/* calc cursor line base	*/
        virtcol = TSS_CC(*screen) + TSS_AW(*screen) *			/* calc virtual column		*/
            ((TSS_CR(*screen) + screen->offset) - linebase);
                
        rowdelta = newsize.height - TSS_AH(*screen);			/* keep distance to bottom	*/
        
        tsAdjustScreenSize(screen, newsize);				/* adjust the screen size	*/
        tsRecalcStoreBuffer(store, TSS_AW(*screen));			/* recalculate store (width)	*/
        
        rowdelta += linebase - TSB_LO(*store, cursorindex);		/* keep relative position	*/

        linebase = TSB_LO(*store, cursorindex);				/* new cursor line base		*/
        TSS_CC(*screen) = virtcol % TSS_AW(*screen);			/* adjust cursor ...		*/
        TSS_CR(*screen) = linebase - screen->offset + 
            (virtcol / TSS_AW(*screen));
        
        tsSmartFocusCursor(store, screen, rowdelta);			/* try to smart focus ...	*/
        tsUnpackStoreToScreen(store, screen, &screen->area);		/* unpack store to screen	*/
        
}

