/*
 *  tstore.h
 *
 *  Created by Herbert Poetzl on Mon Feb 04 2002.
 *  Copyright (c) 2002 H.Poetzl. 
 *
 *  This file is part of MultiTerm.
 *
 *  MultiTerm is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  MultiTerm is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MultiTerm; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef	_TSTORE_INCLUDE_
#define	_TSTORE_INCLUDE_

#include "atext.h"
#include "tview.h"

typedef	struct 	_bit_stream_ptr 	BitSPtr;

typedef struct 	_ts_store_line 		TSStoreLine;
typedef struct 	_ts_store_buffer 	TSStoreBuffer;

typedef struct 	_ts_screen	 	TSScreen;



struct	_bit_stream_ptr {
        unsigned int *ptr;
        short bitpos;
};


struct	_ts_store_line {
        unsigned int offset;
        unsigned short count;
        unsigned int *data;
}	__attribute__ ((packed));

struct	_ts_store_buffer {
        unsigned int cindex;
        unsigned int count;
        unsigned int maxcount;
        unsigned int offset;
        unsigned int freerow;
        unsigned short log2size;
        unsigned short width;
        TSStoreLine *store;
}	__attribute__ ((packed));


struct	_ts_screen {
        ALTextRun run;
        ALTextArea area;
        ALTextCoor cursor;
        ALTextRun buffer;
        unsigned int offset;
        struct _ts_defaults {
            ALTextChar nullchar;
        } defaults;
        struct _ts_update_hints {
            ALTextArea dirtyarea;
            ALTextArea viewport;
            ALTextArea lastline;
            int scrollrows;
        } hints;
        struct _ts_text_screen_flags {
            unsigned wrap:1;
            unsigned scroll:1;
        } flags;
};



/*	tstore procedures	*/



int	tsLineToScreen(
        TSScreen *screen,
        ALTextLine *line);

int	tsDataToScreen(
        TSScreen *screen,
        ALAttribU *attrib,
        const char *data,
        unsigned len);

int	tsStringToScreen(
        TSScreen *screen,
        ALAttribU *attrib,
        const char *string);


void	tsMoveScreenCursor(
        TSScreen *screen,
        int col,
        int row);


void	tsUnpackStoreToTextRun(
        TSStoreBuffer *store, 
        ALTextRun *run,
        ALTextArea *area,
        int offset);

void	tsUnpackStoreToScreen(
        TSStoreBuffer *store, 
        TSScreen *screen,
        ALTextArea *area);

void	tsUpdateStoreFromScreen(
        TSStoreBuffer *store, 
        TSScreen *screen,
        ALTextArea *area,
        int *cursorindex);


int	tsReserveStoreLines(
        TSStoreBuffer *store,
        int count);

int	tsFreeStoreLines(
        TSStoreBuffer *store,
        int index);


int	tsLineForOffset(
        TSStoreBuffer *store,
        int offset);

int	tsStoreIndexForLine(
        TSStoreBuffer *store,
        int line);

int	tsStoreIndexForOffset(
        TSStoreBuffer *store,
        int offset);

void	tsRecalcStoreBuffer(
        TSStoreBuffer *store,
        int width);



void	tsAdjustScreenSize(
        TSScreen *screen,
        TVSize newsize);

void   	tsSmartFocusCursor(
        TSStoreBuffer *store,
        TSScreen *screen,
        int rowpan);

void	tsAdjustTerminalSize(
        TSStoreBuffer *store,
        TSScreen *screen, 
        TVSize newsize);

#endif 	/* _TSTORE_INCLUDE_ */

