/*
 *  tview.c
 *
 *  Created by Herbert Poetzl on Mon Feb 04 2002.
 *  Copyright (c) 2002 H.Poetzl. 
 *
 *  This file is part of MultiTerm.
 *
 *  MultiTerm is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  MultiTerm is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MultiTerm; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "tview.h"
#include "tviewdef.h"

#include "atext.h"
#include "atextdef.h"

#include "genhelper.h"

#include <stdlib.h>


#define	MIX(a,b,x)	(((a)*(x) + (b)*(256-(x))) >> 8)

#define	UMIX(a,b,x)	(((a)>>8)*(x) + ((b)>>8)*(256-(x))) 	



int	tvResizeBitMap(
        TVBitMap *bitmap,
        TVSize size)
{
        if (bitmap->data) 
            bitmap->data = realloc(bitmap->data, TVB_CSIZE(size));
        else bitmap->data = malloc(TVB_CSIZE(size));
        bitmap->size = size;
        bitmap->rowstride = TVB_RSIZE(*bitmap);
        return 0;		/* fix me */
}


void	tvBlitTextChar(
        unsigned char *src,
        int srs,
        TVCharCell *cell,
        TVColor *colors,
        unsigned char *dst,
        int drs)
{
        unsigned char *sp, *dp;
        short x, y, xs, xm, xe, ys, ym, ye;
        TVColorU fg, bg;
        
        fg.comp = colors[1];
        bg.comp = colors[0];
        
        xs = cell->border.left;
        xe = cell->border.right;
        xm = cell->size.width;
        
        ys = cell->border.top;
        ye = cell->border.bottom;
        ym = cell->size.height;
                
        for (y=0; y<ym; y++) {
            sp = src; dp = dst;
            for (x=0; x<xm; x++) {
                unsigned v;
                if ((x < xs) || (y < ys) || 
                    (x >= xe) || (y >= ye) || 
                    ((v = *sp++) == 0)) {
                    *(unsigned *)dp = bg.all;
                } else {
                    *(unsigned *)dp = UMIX(fg.all, bg.all, v);
                }
                dp += 4;
            }
            if (y >= ys) src += srs; 
            dst += drs;
        }
}        
        
void	tvBlitTextRun(
        TVTextOpts *opts, 
        TVBitMap *dest,
        ALTextChar *run, 
        int count)
{
        unsigned char *sp, *dp, first;
        ALAttribU lcha, cha;
        TVColor colors[2];
        TVCharCell cell;
        TVCharSet set;
        div_t divr;
        
        dp = dest->data;
        first = 1;
        while (count--) {
            unsigned int ch = ALC_ASC(*run);
            int ci;
            
            cha.all = (ALC_ALL(*run) & opts->mask.all) 
                ^ opts->xor.all;
            if ((cha.all != lcha.all) || first) {
                colors[0] = opts->colors[cha.detail.color.back];
                colors[1] = opts->colors[cha.detail.color.text];	
                                                                /* fix bright */
                set = *TVO_SR(*opts, cha.detail.charset);	/* fix bold */
                
                cell = opts->cell;
                TVC_BR(cell) = TVS_BL(*opts) + 
                    MIN(TVS_BR(*opts)-TVS_BL(*opts), 
                        TVS_BR(set)-TVS_BL(set));
                TVC_BB(cell) = TVS_BT(*opts) + 
                    MIN(TVS_BB(*opts)-TVS_BT(*opts), 
                        TVS_BB(set)-TVS_BT(set));
                
                first = 0;
            }
            
            ci = ch - set.range.offset;
            if ((ci < 0) | (ci > set.range.count)) ci = 0;
            
            divr = div(ci, set.columns);
            sp = set.data + TVS_BL(set) +
                (divr.rem) * TVS_CW(set) + set.rowstride *
                ((divr.quot) * TVS_CH(set) + TVS_BT(set)); 
            
            tvBlitTextChar(sp, set.rowstride, &cell, 
                colors, dp, dest->rowstride);
            dp += TVS_CW(*opts)*4; run++;
        }
}
