/*
 *  tview.h
 *
 *  Created by Herbert Poetzl on Mon Feb 04 2002.
 *  Copyright (c) 2002 H.Poetzl. 
 *
 *  This file is part of MultiTerm.
 *
 *  MultiTerm is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  MultiTerm is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MultiTerm; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef	_TVIEW_INCLUDE_
#define	_TVIEW_INCLUDE_

#include "atext.h"


typedef	struct 	_tv_point 		TVPoint;
typedef	struct 	_tv_size 		TVSize;
typedef	struct 	_tv_rect 		TVRect;

typedef struct 	_tv_char_cell_border 	TVCellBorder;
typedef struct 	_tv_char_cell 		TVCharCell;
typedef	struct 	_tv_char_range 		TVCharRange;

typedef	struct 	_tv_color 		TVColor;
typedef	union 	_tv_color_union 	TVColorU;

typedef	struct 	_tv_char_set 		TVCharSet;
typedef	struct	_tv_bitmap 		TVBitMap;
typedef	struct 	_tv_opt_sets 		TVOptSets;
typedef	struct 	_tv_text_opts 		TVTextOpts;


struct	_tv_point {
        unsigned short x;
        unsigned short y;
}	__attribute__ ((packed));

struct	_tv_size {
        unsigned short width;
        unsigned short height;
}	__attribute__ ((packed));

struct	_tv_rect {
        TVPoint origin;
        TVSize size;
}	__attribute__ ((packed));


struct 	_tv_char_cell_border {
        unsigned int left:8;
        unsigned int top:8;
        unsigned int right:8;
        unsigned int bottom:8;
} 	__attribute__ ((packed));

struct	_tv_char_cell {
        TVSize size;
        TVCellBorder border;
}	__attribute__ ((packed));

struct _tv_char_range {
        unsigned int offset:16;
        unsigned int count:16;
} 	__attribute__ ((packed));



struct	_tv_color {
        unsigned red:8;
        unsigned green:8;
        unsigned blue:8;
        unsigned _unused:8;
}	__attribute__ ((packed));

union	_tv_color_union {
        TVColor comp;
        unsigned all:32; 
}	__attribute__ ((packed));



struct	_tv_char_set {
        TVCharCell cell;
        TVCharRange range;
        unsigned int columns;
        unsigned int rowstride;
        unsigned char *data;
}	__attribute__ ((packed));

struct	_tv_bitmap {
        TVSize size;
        unsigned int rowstride; 
        unsigned char *data;
}	__attribute__ ((packed));


struct _tv_opt_sets {
        TVCharSet *regular;
        TVCharSet *bold;
        TVCharSet *highlight;
        TVCharSet *underline;
};

struct	_tv_text_opts {
        TVOptSets set[4];
        TVColor *colors;
        TVCharCell cell;
        ALAttribU mask;
        ALAttribU xor;
        unsigned int frame;
};


/*	tview procedures	*/

int	tvResizeBitMap(
        TVBitMap *bitmap,
        TVSize size);

void	tvBlitTextRun(
        TVTextOpts *opts, 
        TVBitMap *dest,
        ALTextChar *run, 
        int count);


#endif 	/* _TVIEW_INCLUDE_ */

