/*
 *  ptyhelper.c
 *
 *  Created by Herbert Poetzl on Fri Jan 25 2002.
 *  Copyright (c) 2002 H.Poetzl. 
 *
 *  This file is part of MultiTerm.
 *
 *  MultiTerm is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  MultiTerm is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MultiTerm; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <util.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <grp.h>

#include "ptyhelper.h"



const char ptym[] = "/dev/pty";
const char ptys[] = "/dev/tty";

#define	PTYM_LEN	(sizeof(ptym)-1 + 2)
#define PTYS_LEN	(sizeof(ptys)-1 + 2)

#define	TTY_GROUP(gid)					\
        {						\
            struct group *grptr;			\
            if ( (grptr = getgrnam("tty")) != NULL)	\
                gid = grptr->gr_gid;			\
            else					\
                gid = -1;  /* not in group file */	\
        }						\





int	forkPTYSlave(
        int fd[2],
        char **slavePath,
        const char *cmd,
        char *const argv[],
        char *const envp[]) 
{
        pid_t pid;
        gid_t gid;
        struct termios *termp = 0;
        struct winsize *winp = 0;
        char *sp;
        
        sp = malloc(PTYS_LEN);
        if (slavePath) *slavePath = sp;
        pid = forkpty(&fd[0], sp, termp, winp);
        if (pid == 0) {						/* child */
                        
            TTY_GROUP(gid);

            /* following two functions don't work unless we're root */
            chown(sp, getuid(), gid);
            chmod(sp, S_IRUSR | S_IWUSR | S_IWGRP);

            write(1, "stdout test output\n", 19);
            write(2, "stderr test output\n", 19);
            write(1, "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n", 61);
            write(1, "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n", 60);
            write(1, "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n", 59);
            write(1, "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n", 58);
            write(1, "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n", 57);
            write(1, "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n", 56);
            write(1, "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n", 55);
            execve (cmd, argv, envp);
            exit(-1);		
            
        } else if (pid > 0) {					/* parent */
            return pid;
            
        } else {						/* failure */
            perror("forkpty():");
            return -1;
        }
}

const
char *	defaultShellPath(void) 
{
        struct passwd *pwd;
        
        pwd = getpwuid(getuid());
        return pwd->pw_shell;
}
